package com.muyu.event.process.consumer;

import com.rabbitmq.client.Channel;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * @Author: zi run
 * @Date 2024/10/2 10:35
 * @Description 测试RabbitMQ消费者
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class TestRabbitMQConsumer {

    /**
     * redis服务实例
     */
    private final StringRedisTemplate redisTemplate;


    /**
     * 队列名称
     */
    private static final String queueName = "test-rabbitmq";

    /**
     * redis缓存键名
     */
    private static final String testRabbitMQKey = "test-rabbitmq";

    /**
     * 消费者方法，用于处理从RabbitMQ队列中接收到的消息
     *
     * @param msg 消息内容，类型为String
     * @param message 原始消息对象，包含消息的属性信息
     * @param channel RabbitMQ通道，用于进行消息确认和拒绝操作
     */
    @RabbitListener(queuesToDeclare = @Queue(name = queueName))
    public void consumer(String msg, Message message, Channel channel) {
        String messageId = message.getMessageProperties().getMessageId();
        try {
            Long count = redisTemplate.opsForSet().add(testRabbitMQKey, messageId);

            if (count != null && count.intValue() == 1) {
                log.info("测试RabbitMQ消费者获取到消息，消息内容：{}", msg);
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                log.info("测试RabbitMQ消费者消费消息成功！");
            }
        } catch (Exception e) {
            redisTemplate.opsForSet().remove(testRabbitMQKey, messageId);
            log.error("测试RabbitMQ消费者消费消息异常，消息内容：{}，异常信息：{}", msg, e.getMessage());

            try {
                channel.basicReject(message.getMessageProperties().getDeliveryTag(), true);
            } catch (IOException ex) {
                log.error("测试RabbitMQ消费者回退消息异常，消息内容：{}， 异常信息：{}", msg, ex.getMessage());
            }
        }
    }

}
