package com.muyu.event.process.consumer;

import com.muyu.common.core.constant.KafkaConstants;
import com.muyu.event.process.basic.EventPublisher;
import com.muyu.event.process.event.DataProcessEvent;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextClosedEvent;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

/**
 * @Author: zi run
 * @Date 2024/9/29 23:23
 * @Description 车辆消费者
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class VehicleConsumer implements ApplicationRunner, ApplicationListener<ContextClosedEvent> {

    /**
     * kafka消费者
     */
    private final KafkaConsumer<String, String> kafkaConsumer;

    /**
     * 事件发布者
     */
    private final EventPublisher eventPublisher;

    /**
     * 设定固定大小的线程池，线程数量与当前可用的处理器核心数相同
     */
    private final ExecutorService executorService =
            Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());

    /**
     * 启动消费者并监听Kafka主题，处理接收到的消息
     * @param args 应用程序参数
     * @throws Exception 如果启动过程中发生错误
     */
    @Async
    @Override
    public void run(ApplicationArguments args) throws Exception {
        log.info("启动线程监听Topic: {}", KafkaConstants.MESSAGE_PARSING);
        List<String> topics = Collections.singletonList(KafkaConstants.MESSAGE_PARSING);
        kafkaConsumer.subscribe(topics);
        while (true) {
            ConsumerRecords<String, String> consumerRecords = kafkaConsumer.poll(Duration.ofMillis(200));
            consumerRecords.forEach(consumerRecord -> executorService.submit(() -> handleRecord(consumerRecord)));
        }
    }

    /**
     * 处理接收到的Kafka消息记录，将其转换为事件并发布。
     * @param consumerRecord Kafka消费者记录
     */
    private void handleRecord(ConsumerRecord<String, String> consumerRecord) {
        String message = consumerRecord.value();
        log.info("接收到车辆报文数据，内容：{}", message);
        log.info("------------------------------------------------");
        eventPublisher.publish(new DataProcessEvent(this, message));
    }

    /**
     * 处理应用关闭事件，关闭线程池和Kafka消费者
     * @param event 应用上下文关闭事件
     */
    @Override
    public void onApplicationEvent(ContextClosedEvent event) {
        log.info("关闭线程池和Kafka消费者");

        try {
            executorService.shutdown();
            if (!executorService.awaitTermination(60, TimeUnit.SECONDS)) {
                executorService.shutdownNow();
            }
        } catch (InterruptedException e) {
            log.error("线程池关闭被中断，强制关闭", e);
            executorService.shutdownNow();
            Thread.currentThread().interrupt();
        }

        try {
            kafkaConsumer.close();
        } catch (Exception e) {
            log.error("关闭Kafka消费者时发生错误", e);
        }
    }
}