package com.muyu.event.process.iotdb.basic.service;

import com.muyu.event.process.iotdb.domain.dto.IoTDbRecordAble;
import com.muyu.event.process.iotdb.domain.dto.MeasurementSchemaValuesDTO;
import org.apache.iotdb.common.rpc.thrift.TAggregationType;
import org.apache.iotdb.isession.SessionDataSet;
import org.apache.iotdb.tsfile.file.metadata.enums.TSDataType;
import org.apache.iotdb.tsfile.write.record.Tablet;
import org.apache.iotdb.tsfile.write.schema.MeasurementSchema;

import java.util.List;
import java.util.Map;

/**
 * @Author: zi run
 * @Date 2024/9/28 23:37
 * @Description IoTDB基准业务层
 */
public interface IService {

    /**
     * 插入一个 Tablet 对象到 IoTDB 数据库
     *
     * @param tablet 要插入的 Tablet 对象，包含待写入的数据
     */
    void insertTablet(Tablet tablet);

    /**
     * 将给定的 Tablets 插入到 IoTDB 数据库中。
     *
     * @param tablets 一个 Map，包含要插入的 Tablets
     */
    void insertTablets(Map<String, Tablet> tablets);

    /**
     * 单条数据插入（string类型数据项）
     *
     * @param deviceId     设备名（表名）root.ln.wf01.wt01
     * @param time         时间戳
     * @param measurements 数据项列表
     * @param values       数据项对应值列表
     */
    void insertStringRecord(String deviceId, long time, List<String> measurements, List<String> values);

    /**
     * 单条数据插入（不同类型数据项）
     *
     * @param deviceId     设备名（表名）root.ln.wf01.wt01
     * @param time         时间戳
     * @param measurements 数据项列表
     * @param types        数据项对应类型列表
     * @param values       数据项对应值列表
     */
    void insertRecord(String deviceId, long time, List<String> measurements,
                      List<TSDataType> types, List<Object> values);

    /**
     * 多个设备多条数据插入（string类型数据项）
     *
     * @param deviceIds        多个设备名（表名）root.ln.wf01.wt01
     * @param times            时间戳的列表
     * @param measurementsList 数据项列表的列表
     * @param valuesList       数据项对应值列表的列表
     */
    void insertStringRecords(List<String> deviceIds, List<Long> times,
                             List<List<String>> measurementsList, List<List<String>> valuesList);

    /**
     * 多个设备多条数据插入（不同类型数据项）
     *
     * @param deviceIds        多个设备名（表名））root.ln.wf01.wt01
     * @param times            时间戳的列表
     * @param measurementsList 数据项列表的列表
     * @param typesList        数据项对应类型列表的列表
     * @param valuesList       数据项对应值列表的列表
     */
    void insertRecords(List<String> deviceIds, List<Long> times, List<List<String>> measurementsList,
                       List<List<TSDataType>> typesList, List<List<Object>> valuesList);

    /**
     * 单个设备多条数据插入（string类型数据项）
     *
     * @param deviceId         单个设备名（表名））root.ln.wf01.wt01
     * @param times            时间戳的列表
     * @param measurementsList 数据项列表的列表
     * @param valuesList       数据项对应值列表的列表
     */
    void insertStringRecordsOfOneDevice(String deviceId, List<Long> times,
                                        List<List<String>> measurementsList, List<List<String>> valuesList);

    /**
     * 单个设备多条数据插入（不同类型数据项）
     *
     * @param deviceId         单个设备名（表名））root.ln.wf01.wt01
     * @param times            时间戳的列表
     * @param measurementsList 数据项列表的列表
     * @param typesList        数据项对应类型列表的列表
     * @param valuesList       数据项对应值列表的列表
     */
    void insertRecordsOfOneDevice(String deviceId, List<Long> times, List<List<String>> measurementsList,
                                  List<List<TSDataType>> typesList, List<List<Object>> valuesList);

    /**
     * 删除数据（删除一个时间序列在某个时间点前或这个时间点的数据）
     *
     * @param path    单个字段 root.ln.wf01.wt01.temperature
     * @param endTime 删除时间点
     */
    void deleteData(String path, long endTime);

    /**
     * 删除数据（删除多个时间序列在某个时间点前或这个时间点的数据）
     *
     * @param paths   多个字段（表名）） root.ln.wf01.wt01.temperature
     * @param endTime 删除时间点
     */
    void deleteData(List<String> paths, long endTime);

    /**
     * 数据查询（时间序列原始数据范围查询，时间范围为左闭右开区间，包含开始时间但不包含结束时间）
     *
     * @param paths     多个字段（表名）） root.ln.wf01.wt01.temperature
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @param outTime   超时时间
     * @return SessionDataSet  (Time,paths)
     */
    SessionDataSet executeRawDataQuery(List<String> paths, long startTime, long endTime, long outTime);

    /**
     * 数据查询（时间序列原始数据范围查询，时间范围为左闭右开区间，包含开始时间但不包含结束时间）
     *
     * @param paths     多个字段（表名），例如："root.ln.wf01.wt01.temperature"
     * @param startTime 查询数据的起始时间（包含该时间点）
     * @param endTime   查询数据的结束时间（不包含该时间点）
     * @param outTime   超时时间，单位为毫秒，表示查询的最长等待时间
     * @param clazz     返回数据对应的对象类型，要求对象属性与数据库字段名一致
     * @param <T>      返回数据的对象类型泛型
     * @return        查询结果的对象列表，如果查询失败则返回 null
     */
    <T> List<T> executeRawDataQuery(List<String> paths, long startTime, long endTime, long outTime,
                                    Class<? extends IoTDbRecordAble> clazz);

    /**
     * 最新点查询(查询最后一条时间戳大于等于某个时间点的数据)
     *
     * @param paths    多个字段（表名）） root.ln.wf01.wt01.temperature
     * @param lastTime 结束时间
     * @return SessionDataSet
     */
    SessionDataSet executeLastDataQuery(List<String> paths, long lastTime);

    /**
     * 最新点查询(查询最后一条时间戳大于等于某个时间点的数据)
     *
     * @param paths    多个字段（表名）） root.ln.wf01.wt01.temperature
     * @param lastTime 结束时间
     * @param clazz    返回数据对应的对象（对象属性必须与字段名对应）
     * @return 查询结果的对象列表，如果查询失败则返回 null
     * @param <T> 返回数据的对象类型泛型
     */
    <T> List<T> executeLastDataQuery(List<String> paths, long lastTime, Class<? extends IoTDbRecordAble> clazz);

    /**
     * 最新点查询（快速查询单设备下指定序列最新点）
     *
     * @param db               root.ln.wf01
     * @param device           root.ln.wf01.wt01
     * @param sensors          temperature，status(字段名)
     * @param isLegalPathNodes true(避免路径校验)
     * @return SessionDataSet
     */
    SessionDataSet executeLastDataQueryForOneDevice(String db, String device,
                                                    List<String> sensors, boolean isLegalPathNodes);

    /**
     * 查询单个设备的最新数据（获取指定设备的最新传感器数据）
     *
     * @param db               root.ln.wf01
     * @param device           root.ln.wf01.wt01
     * @param sensors          temperature，status(字段名)
     * @param isLegalPathNodes true(避免路径校验)
     * @param clazz            返回数据对应的对象（对象属性必须与字段名对应）
     * @return 查询结果的对象列表，如果查询失败则返回 null
     * @param <T> 返回数据的对象类型泛型
     */
    <T> List<T> executeLastDataQueryForOneDevice(String db, String device, List<String> sensors,
                                                 boolean isLegalPathNodes, Class<? extends IoTDbRecordAble> clazz);

    /**
     * 聚合查询
     *
     * @param paths        多个字段（表名）） root.ln.wf01.wt01.temperature
     * @param aggregations 聚合操作 TAggregationType.SUM,TAggregationType.COUNT
     * @return SessionDataSet
     */
    SessionDataSet executeAggregationQuery(List<String> paths, List<TAggregationType> aggregations);

    /**
     * 聚合查询(时间序列原始数据范围查询，时间范围为左闭右开区间，包含开始时间但不包含结束时间)
     *
     * @param paths        多个字段（表名）） root.ln.wf01.wt01.temperature
     * @param aggregations 聚合操作 TAggregationType.SUM,TAggregationType.COUNT
     * @param startTime    开始时间(包含)
     * @param endTime      结束时间
     * @return SessionDataSet
     */
    SessionDataSet executeAggregationQuery(List<String> paths, List<TAggregationType> aggregations,
                                           long startTime, long endTime);

    /**
     * 聚合查询(支持按照时间区间分段查询)
     *
     * @param paths        多个字段（表名）） root.ln.wf01.wt01.temperature
     * @param aggregations 聚合操作 TAggregationType.SUM,TAggregationType.COUNT
     * @param startTime    开始时间(包含)
     * @param endTime      结束时间
     * @param interval 查询的时间间隔（单位为毫秒）
     * @return SessionDataSet
     */
    SessionDataSet executeAggregationQuery(List<String> paths, List<TAggregationType> aggregations,
                                           long startTime, long endTime, long interval);

    /**
     * 聚合查询(支持按照时间区间分段查询)
     *
     * @param paths        多个字段（表名）） root.ln.wf01.wt01.temperature
     * @param aggregations 聚合操作 TAggregationType.SUM,TAggregationType.COUNT
     * @param startTime    开始时间(包含)
     * @param endTime      结束时间
     * @param interval     查询的时间间隔（单位为毫秒）
     * @param slidingStep  滑动步长（单位为毫秒）
     * @return SessionDataSet
     */
    SessionDataSet executeAggregationQuery(List<String> paths, List<TAggregationType> aggregations,
                                           long startTime, long endTime, long interval, long slidingStep);

    /**
     * SQL查询
     *
     * @param sql SQL查询语句，支持IotDB的查询语法
     * @return 返回查询结果的 SessionDataSet，如果执行失败则返回 null
     */
    SessionDataSet executeQueryStatement(String sql);


    /**
     * SQL非查询
     *
     * @param sql SQL查询语句，支持IotDB的查询语法
     */
    void executeNonQueryStatement(String sql);

    /**
     * 封装处理数据
     *
     * @param sessionDataSet 包含查询结果的SessionDataSet对象
     * @param titleList 列标题列表，用于映射字段名称
     * @return 返回封装后的数据列表，每个 Map 代表一行数据，键为列名，值为对应的字段值
     */
    List<Map<String, Object>> packagingMapData(SessionDataSet sessionDataSet, List<String> titleList);

    /**
     * 封装处理数据（不支持聚合查询）
     *
     * @param sessionDataSet 查询返回的结果集
     * @param titleList      查询返回的结果集内的字段名
     * @param clazz          返回数据对应的对象（对象属性必须与字段名对应）
     * @return 返回封装后的对象列表，每个对象对应一行结果集数据
     * @param <T> 返回对象的类型
     */
    <T> List<T> packagingObjectData(SessionDataSet sessionDataSet, List<String> titleList,
                                    Class<? extends IoTDbRecordAble>  clazz);

    /**
     * 根据对象构建MeasurementSchemas
     *
     * @param obj 要从中提取字段信息的对象
     * @return 返回一个包含 MeasurementSchema 的列表
     */
    List<MeasurementSchema> buildMeasurementSchemas(Object obj);

    /**
     * 根据对象构建MeasurementSchemaValuesDTO
     *
     * @param obj 要从中提取字段信息和对应值的对象
     * @return  MeasurementSchemaValuesDTO 对象
     */
    MeasurementSchemaValuesDTO buildMeasurementSchemasAndValues(Object obj);
}