package com.muyu.event.process.listener;

import com.alibaba.fastjson2.JSONObject;
import com.muyu.event.process.basic.BasicEvent;
import com.muyu.event.process.basic.BasicEventListener;
import com.muyu.event.process.iotdb.service.IoTDBService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author: zi run
 * @Date 2024/9/29 22:12
 * @Description 向IoTDB插入数据事件监听器
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class IoTDBInsertDataEventListener implements BasicEventListener<String> {

    /**
     * IoTDB业务层
     */
    private final IoTDBService ioTDBService;

    /**
     * 设备名（表名）
     */
    private static final String DEVICE_ID = "root.vehicle";

    /**
     * 处理接收到的事件，将数据插入到 IoTDB
     *
     * @param event 接收到的事件，包含需要插入的数据
     */
    @Override
    public void onEvent(BasicEvent<String> event) {
        log.info("触发向IoTDB插入数据事件监听器……");
        JSONObject data = JSONObject.parseObject(event.getData());
        List<String> keyList = extractKeys(data);
        List<String> valueList = extractValues(data);
        ioTDBService.insertStringRecord(DEVICE_ID, System.currentTimeMillis(), keyList, valueList);
    }

    /**
     * 从给定的JSONObject中提取所有的键
     *
     * @param data 要提取键的JSONObject
     * @return 键的列表
     */
    private List<String> extractKeys(JSONObject data) {
        return data.keySet().stream().toList();
    }

    /**
     * 从给定的 JSONObject 中提取所有的值，并将其转换为字符串
     *
     * @param data 要提取值的JSONObject
     * @return 值的列表，以字符串形式表示
     */
    private List<String> extractValues(JSONObject data) {
        return data.values().stream()
                .map(Object::toString)
                .collect(Collectors.toList());
    }
}